@@ -39,6 +39,8 @@
 #' @param response (optional) a character vector or list specifying the response
 #'     headers to return e.g. \code{c("date", "server")} or \code{list("Date", "Server")}.
 #'     These are case-insensitive and will return NULL if not present.
+#' @param timeout (optional) integer value in milliseconds after which the
+#'     transaction times out if not yet complete.
 #' @param pem (optional) applicable to secure HTTPS sites only. The path to a
 #'     file containing X.509 certificate(s) in PEM format, comprising the
 #'     certificate authority certificate chain (and revocation list if present).
@@ -63,9 +65,17 @@
 #'     and 'recvAio') (invisibly).
 #'
 #' @examples
-#' ncurl("https://httpbin.org/get", response = c("date", "server"))
-#' ncurl("http://httpbin.org/put",,,,"PUT", list(Authorization = "Bearer APIKEY"), "hello world")
-#' ncurl("http://httpbin.org/post",,,,"POST", c(`Content-Type` = "application/json"),'{"k":"v"}')
+#' ncurl("https://www.cam.ac.uk/", response = c("date", "server"), timeout = 1000L)
+#' ncurl("http://httpbin.org/put",
+#'       method = "PUT",
+#'       headers = list(Authorization = "Bearer APIKEY"),
+#'       data = "hello world",
+#'       timeout = 2000L)
+#' ncurl("http://httpbin.org/post",
+#'       method = "POST",
+#'       headers = c(`Content-Type` = "application/json"),
+#'       data = '{"k":"v"}',
+#'       timeout = 2000L)
 #'
 #' @export
 #'
@@ -77,10 +87,11 @@ ncurl <- function(url,
                   headers = NULL,
                   data = NULL,
                   response = NULL,
+                  timeout = NULL,
                   pem = NULL)
   if (async)
-    data <- .Call(rnng_ncurl_aio, url, convert, method, headers, data, pem, environment()) else
-      .Call(rnng_ncurl, url, convert, follow, method, headers, data, response, pem)
+    data <- .Call(rnng_ncurl_aio, url, convert, method, headers, data, timeout, pem, environment()) else
+      .Call(rnng_ncurl, url, convert, follow, method, headers, data, response, timeout, pem)
 
 #' ncurl Session
 #'
@@ -97,7 +108,7 @@ ncurl <- function(url,
 #'     else an 'errorValue'.
 #'
 #' @examples
-#' s <- ncurl_session("https://httpbin.org/get", response = "date", timeout = 1000L)
+#' s <- ncurl_session("https://httpbin.org/get", response = "date", timeout = 2000L)
 #' s
 #' if (!is_error_value(s)) transact(s)
 #' if (!is_error_value(s)) close(s)
@@ -141,4 +152,3 @@ transact <- function(session) .Call(rnng_ncurl_transact, session)
 #' @export
 #'
 close.ncurlSession <- function(con, ...) invisible(.Call(rnng_ncurl_session_close, con))
-
