@@ -10,6 +10,7 @@ nanotestxp <- function(x) invisible(typeof(x) == "externalptr" || stop())
 nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop())
 
+nanotest(is.character(ver <- nanonext_version()) && length(ver) == 1L)
 n <- nano("pair", listen = "inproc://nanonext", autostart = FALSE)
 n1 <- nano("pair", dial = "inproc://nanonext", autostart = FALSE)
 nanotest(inherits(n, "nanoObject"))
@@ -170,7 +171,7 @@ nanotestn(stat(rep, "nonexistentstat"))
 nanotestnano(req$opt("req:resend-time", 1000))
 nanotest(req$opt("req:resend-time") == 1000L)
 nanotesterr(req$opt("none"), "supported")
-nanotestxp(req$context_open(verify = FALSE))
+nanotestxp(req$context_open())
 nanotest(inherits(req$context, "nanoContext"))
 nanotest(inherits(req$context, "nano"))
 nanotest(is.integer(req$context$id))
@@ -222,12 +223,14 @@ nanotestz(pipe_notify(rep, cv))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
 nanotesterr(recv_aio_signal(err, timeout = 500, cv = cv))
-nanotesterr(request_signal(req$context, "test", cv = err), "valid")
+nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
 nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
 nanotesterr(wait(err), "valid")
 nanotesterr(until(err, 10), "valid")
 nanotesterr(cv_value(err), "valid")
 nanotesterr(cv_reset(err), "valid")
+nanotesterr(cv_signal(err), "valid")
+nanotesterr(timed_signal(err, 1000), "valid")
 nanotesterr(pipe_notify(err, cv), "valid Socket")
 nanotesterr(pipe_notify(rep, err), "valid Condition Variable")
 nanotestz(req$context_close())
@@ -263,6 +266,13 @@ nanotestz(sub$close())
 nanotestz(pub$close())
 nanotest(wait(cv))
 nanotest(wait(cv2))
+nanotestxp(sig <- timed_signal(cv, time = 50L))
+nanotest(!wait(cv))
+nanotestxp(sig <- timed_signal(cv2, time = 50, flag = FALSE))
+nanotest(wait(cv2))
+nanotestp(sig)
+nanotestn(cv_signal(cv))
+nanotest(cv_value(cv) == 1L)
 
 surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2")
 nanotestp(surv)
@@ -304,8 +314,8 @@ nanotestnn(ncurl("http://www.cam.ac.uk/"))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "date"))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date")))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
-nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server"))
-nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server")))
+nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
+nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
 relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = "server", async = TRUE)
 nanotest(is.integer(call_aio(relo)$status))
 relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = list("server"), async = TRUE)
@@ -315,7 +325,7 @@ nanotest(is.integer(call_aio(put1)$status))
 nanotestnn(put1$headers)
 nanotestnn(put1$raw)
 nanotestnn(put1$data)
-put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"))
+put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
 nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
@@ -324,10 +334,10 @@ nanotest(is_error_value(ncurl("http", async = TRUE)$data))
 haio <- ncurl("https://i.i", async = TRUE)
 nanotest(is_error_value(call_aio(haio)$raw))
 nanotestp(haio$data)
-sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 2000L)
+sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
-sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 2000)
+sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
@@ -396,7 +406,7 @@ nanotest(is_error_value(recv(s2)))
 nanotestz(unlock(s))
 nanotesterr(unlock(cv), "valid Socket")
 cv <- cv()
-pipe_notify(s, cv = cv, flag = FALSE)
+nanotestz(pipe_notify(s, cv = cv, flag = FALSE))
 s3 <- socket("bus", dial = "inproc://nanolock")
 nanotestz(send(s, "test"))
 nanotestnn(recv(s3))
@@ -418,6 +428,7 @@ nanotesterr(dial(fakesock, autostart = FALSE), "valid Socket")
 nanotesterr(listen(fakesock), "valid Socket")
 nanotesterr(listen(fakesock, autostart = FALSE), "valid Socket")
 nanotesterr(context(fakesock), "valid Socket")
+nanotesterr(.context(fakesock), "valid Socket")
 nanotesterr(stat(fakesock, "pipes"), "valid Socket")
 nanotesterr(close(fakesock), "valid Socket")
 nanotest(.unresolved(fakesock) == FALSE)
@@ -468,4 +479,3 @@ nanotest(base64dec(base64enc("test")) == "test")
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
 nanotest(is.integer(unserialize(base64dec(base64enc(c(1L, 2L)), convert = FALSE))))
 nanotesterr(base64dec("__"), "not valid base64")
-
